(function ($, Drupal, drupalSettings) {
  
  "use strict";

  /**
   * Attaches the JS countdown behavior
   */
  Drupal.behaviors.jsCountdownTimer = {
    attach: function (context) {
      if (drupalSettings.countdown) {
        $.each(drupalSettings.countdown, function (index, value) {
          let ts = new Date(value.unixtimestamp * 1000);
          let ts_event = new Date(value.unixtimestamp_event * 1000);
          let current_date = Math.floor((new Date()) / 1000);
          let form_class = value.form_class + ' button';
          let note_display = $('#' + value.timer_display_element_note);
          let info_display = $('#' + value.timer_display_element_info);
          let event_start_minutes = value.event_start_minutes;
          let event_start_prefix = Drupal.t('Event starts in');
          let event_start_suffix = Drupal.t('minutes');
          let event_start_line = event_start_prefix + ' ' + event_start_minutes + ' ' + event_start_suffix;
          let event_start_info = Drupal.t('Event open to join!');
          // CountDown Element
          $(context).find('#' + value.timer_display_element).once(value.timer_display_element).countdown({
            timestamp: ts_event,
            font_size: value.fontsize,
            callback: function (weeks, days, hours, minutes, seconds) {
              let dateStrings = new Array();
              dateStrings['@weeks'] = Drupal.formatPlural(weeks, '@count week', '@count weeks');
              dateStrings['@days'] = Drupal.formatPlural(days, '@count day', '@count days');
              dateStrings['@hours'] = Drupal.formatPlural(hours, '@count hour', '@count hours');
              dateStrings['@minutes'] = Drupal.formatPlural(minutes, '@count minute', '@count minutes');
              dateStrings['@seconds'] = Drupal.formatPlural(seconds, '@count second', '@count seconds');
              let message_headline = Drupal.t('Event starts in ');
              if (days == 0) {
                if (hours == 0) {
                  var message = Drupal.t('@minutes.', dateStrings);
                } 
                else {
                  var message = Drupal.t('@hours, @minutes.', dateStrings);
                }
              } 
              else {
                var message = Drupal.t('@days, @hours, @minutes.', dateStrings);
              }
              note_display.html('<div>' + message_headline + '</div><div>' + message + '</div>');
              //
            }
          });
          // Timer
          $(context).find('#' + value.timer_element).once(value.timer_element).countdown({
            timestamp: ts,
            font_size: value.fontsize,
            callback: function (weeks, days, hours, minutes, seconds) {
              let done = weeks + days + hours + minutes + seconds;
              let dateStrings = new Array();
              dateStrings['@weeks'] = Drupal.formatPlural(weeks, '@count week', '@count weeks');
              dateStrings['@days'] = Drupal.formatPlural(days, '@count day', '@count days');
              dateStrings['@hours'] = Drupal.formatPlural(hours, '@count hour', '@count hours');
              dateStrings['@minutes'] = Drupal.formatPlural(minutes, '@count minute', '@count minutes');
              dateStrings['@seconds'] = Drupal.formatPlural(seconds, '@count second', '@count seconds');
              if (done == 0) {
                note_display.hide();
                $('#' + value.timer_element).hide();
                info_display.html('<div class="text-center">' + event_start_info + '</div>');
                $('form.' + form_class).removeClass('visually-hidden');                
              }
            }
          });
        });
      }
      // Timer Element
      if (drupalSettings.timer) {
        $.each(drupalSettings.timer, function (index, value) {
          let ts = new Date(value.unixtimestamp * 1000);
          let form_class = value.form_class + ' button';
          $(context).find('#' + value.timer_element).once(value.timer_element).countdown({
            timestamp: ts,
            font_size: value.fontsize,
            callback: function (weeks, days, hours, minutes, seconds) {
              let dateStrings = new Array();
              dateStrings['@weeks'] = Drupal.formatPlural(weeks, '@count week', '@count weeks');
              dateStrings['@days'] = Drupal.formatPlural(days, '@count day', '@count days');
              dateStrings['@hours'] = Drupal.formatPlural(hours, '@count hour', '@count hours');
              dateStrings['@minutes'] = Drupal.formatPlural(minutes, '@count minute', '@count minutes');
              dateStrings['@seconds'] = Drupal.formatPlural(seconds, '@count second', '@count seconds');
              let message = Drupal.t('@hours : @minutes : @seconds', dateStrings);
              if (message === '0 hours : 0 minutes : 0 seconds') {
                $('form.' + form_class).addClass('visually-hidden');
                $('.block-social-virtual-event-bbb-join-button-block').addClass('visually-hidden');
              }

            }
          });
        });
      }
    }
  };
})(jQuery, Drupal, drupalSettings);
